﻿//---------------------------------------------------------------------------
//
//   Measure
//
//          Copyright(C) 1999-2012, O.Mizno
//
//     Sample of Select plug-in for measuring a length of a selected line.
//     Created DLL must be installed in "Plugins\Select" directory.
//
//    　選択したラインの長さを測る選択プラグインのサンプル。
//    　作成したDLLは"Plugins\Select"フォルダに入れる必要がある。
//
//
//---------------------------------------------------------------------------

#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <stdio.h>
#include <math.h>
#include "MQPlugin.h"

BOOL MeasureLength(MQDocument doc);


//---------------------------------------------------------------------------
//  DllMain
//---------------------------------------------------------------------------
BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					 )
{
	//プラグインとしては特に必要な処理はないので、何もせずにTRUEを返す
    return TRUE;
}

//---------------------------------------------------------------------------
//  MQGetPlugInID
//    プラグインIDを返す。
//    この関数は起動時に呼び出される。
//---------------------------------------------------------------------------
MQPLUGIN_EXPORT void MQGetPlugInID(DWORD *Product, DWORD *ID)
{
	// プロダクト名(制作者名)とIDを、全部で64bitの値として返す
	// 値は他と重複しないようなランダムなもので良い
	*Product = 0x56A31D20;
	*ID      = 0x1E53AC5F;
}

//---------------------------------------------------------------------------
//  MQGetPlugInName
//    プラグイン名を返す。
//    この関数は[プラグインについて]表示時に呼び出される。
//---------------------------------------------------------------------------
MQPLUGIN_EXPORT const char *MQGetPlugInName(void)
{
	// プラグイン名
	return "Measure           Copyright(C) 1999-2012, O.Mizno";
}

//---------------------------------------------------------------------------
//  MQGetPlugInType
//    プラグインのタイプを返す。
//    この関数は起動時に呼び出される。
//---------------------------------------------------------------------------
MQPLUGIN_EXPORT int MQGetPlugInType(void)
{
	// 選択部変形用プラグインである
	return MQPLUGIN_TYPE_SELECT;
}

//---------------------------------------------------------------------------
//  MQEnumString
//    ポップアップメニューに表示される文字列を返す。
//    この関数は起動時に呼び出される。
//---------------------------------------------------------------------------
MQPLUGIN_EXPORT const char *MQEnumString(int index)
{
	switch(index){
	case 0: return "Measure length of lines";
	}
	return NULL;
}

//---------------------------------------------------------------------------
//  MQModifySelect
//    メニューから選択されたときに呼び出される。
//---------------------------------------------------------------------------
MQPLUGIN_EXPORT BOOL MQModifySelect(int index, MQDocument doc)
{
	switch(index){
	case 0: return MeasureLength(doc);
	}
	return FALSE;
}

//長さの計測
BOOL MeasureLength(MQDocument doc)
{
	MQSelectLine sel;
	int vert[4];
	MQPoint vec;
	float length = 0.0f;
	char buf[64];

	int obj_count = doc->GetObjectCount();

	// オブジェクトの数だけ繰り返し
	for(sel.object=0; sel.object<obj_count; sel.object++)
	{
		MQObject obj = doc->GetObject(sel.object);
		if(obj == NULL)
			continue;

		// オブジェクト中の面数(正確には配列の大きさ)を得る
		int face_count = obj->GetFaceCount();

		for(sel.face=0; sel.face<face_count; sel.face++)
		{
			// 無効な面(頂点の参照数が0)ならパス
			int ptcount = obj->GetFacePointCount(sel.face);
			if(ptcount == 0)
				continue;

			// ラインの長さを計測
			if(ptcount >= 3) {
				for(sel.line=0; sel.line<ptcount; sel.line++)
					if(doc->IsSelectLine(sel)) {
						obj->GetFacePointArray(sel.face, vert);
						vec = obj->GetVertex(vert[sel.line])
						    - obj->GetVertex(vert[(sel.line+1)%ptcount]);
						length += (float)sqrt(vec.x*vec.x + vec.y*vec.y + vec.z*vec.z);
					}
			} else if(ptcount == 2) {
				sel.line = 0;
				if(doc->IsSelectLine(sel)) {
					obj->GetFacePointArray(sel.face, vert);
					vec = obj->GetVertex(vert[0])
					    - obj->GetVertex(vert[1]);
					length += (float)sqrt(vec.x*vec.x + vec.y*vec.y + vec.z*vec.z);
				}
			}
		}
	}

	// 長さの表示
	sprintf_s(buf, sizeof(buf), "Length = %.2f", length);
	MessageBox(MQ_GetWindowHandle(), buf, "Measure", MB_OK);
	
	return TRUE;
}

