﻿//---------------------------------------------------------------------------
//
//   Separate
//
//          Copyright(C) 1999-2012, O.Mizno
//
//     Sample of Select plug-in for making selected faces scattering.
//     Created DLL must be installed in "Plugins\Select" directory.
//
//    　選択した面をバラバラにする選択プラグインのサンプル。
//    　作成したDLLは"Plugins\Select"フォルダに入れる必要がある。
//
//
//---------------------------------------------------------------------------

#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include "MQPlugin.h"

HINSTANCE hInstance;

BOOL Separate(MQDocument doc);


//---------------------------------------------------------------------------
//  DllMain
//---------------------------------------------------------------------------
BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					 )
{
	//ダイアログボックスの表示に必要なので、インスタンスを保存しておく
	hInstance = (HINSTANCE)hModule;

	//プラグインとしては特に必要な処理はないので、何もせずにTRUEを返す
    return TRUE;
}

//---------------------------------------------------------------------------
//  MQGetPlugInID
//    プラグインIDを返す。
//    この関数は起動時に呼び出される。
//---------------------------------------------------------------------------
MQPLUGIN_EXPORT void MQGetPlugInID(DWORD *Product, DWORD *ID)
{
	// プロダクト名(制作者名)とIDを、全部で64bitの値として返す
	// 値は他と重複しないようなランダムなもので良い
	*Product = 0x56A31D20;
	*ID      = 0x3EA0456F;
}

//---------------------------------------------------------------------------
//  MQGetPlugInName
//    プラグイン名を返す。
//    この関数は[プラグインについて]表示時に呼び出される。
//---------------------------------------------------------------------------
MQPLUGIN_EXPORT const char *MQGetPlugInName(void)
{
	// プラグイン名
	return "Separate faces        Copyright(C) 1999-2012, O.Mizno";
}

//---------------------------------------------------------------------------
//  MQGetPlugInType
//    プラグインのタイプを返す。
//    この関数は起動時に呼び出される。
//---------------------------------------------------------------------------
MQPLUGIN_EXPORT int MQGetPlugInType(void)
{
	// 選択部変形用プラグインである
	return MQPLUGIN_TYPE_SELECT;
}

//---------------------------------------------------------------------------
//  MQEnumString
//    ポップアップメニューに表示される文字列を返す。
//    この関数は起動時に呼び出される。
//---------------------------------------------------------------------------
MQPLUGIN_EXPORT const char *MQEnumString(int index)
{
	switch(index){
	case 0: return "Separate faces";
	}
	return NULL;
}

//---------------------------------------------------------------------------
//  MQModifySelect
//    メニューから選択されたときに呼び出される。
//---------------------------------------------------------------------------
MQPLUGIN_EXPORT BOOL MQModifySelect(int index, MQDocument doc)
{
	switch(index){
	case 0: return Separate(doc);
	}
	return FALSE;
}

struct {
	int segment;
} lathe;

//面をバラバラにする
BOOL Separate(MQDocument doc)
{
	int vert, vert_index[4], new_vert[4];
	MQCoordinate uv[4];
	MQSelectFace sel;
	BOOL result = FALSE;

	int obj_count = doc->GetObjectCount();

	for(sel.object=0; sel.object<obj_count; sel.object++)
	{
		MQObject obj = doc->GetObject(sel.object);
		if(obj == NULL)
			continue;

		int face_count = obj->GetFaceCount();
		int vert_count = obj->GetVertexCount();

		for(sel.face=0; sel.face<face_count; sel.face++)
		{
			int count = obj->GetFacePointCount(sel.face);
			if(count == 0)
				continue;
			
			if(doc->IsSelectFace(sel))
			{
				// 独立した頂点を追加
				obj->GetFacePointArray(sel.face, vert_index);
				for(vert=0; vert<count; vert++)
				{
					new_vert[vert] = obj->AddVertex( obj->GetVertex(vert_index[vert]) );
					// 新しい頂点に属性(頂点カラーなど)をコピー
					obj->CopyVertexAttribute(new_vert[vert], obj, vert_index[vert]);
				}

				// 新しい頂点で構成された面を登録
				int new_face = obj->AddFace(count, new_vert);
				// 属性(材質インデックス、UVマッピング)をコピー
				obj->SetFaceMaterial(new_face, obj->GetFaceMaterial(sel.face));
				obj->GetFaceCoordinateArray(sel.face, uv);
				obj->SetFaceCoordinateArray(new_face, uv);
				// 新しい面を選択
				doc->AddSelectFace(MQSelectFace(sel.object, new_face));
				
				// 元々の面を削除
				obj->DeleteFace(sel.face);

				result = TRUE;
			}
		}
	}
	
	return result;
}

