﻿//---------------------------------------------------------------------------
//
//   VColor
//
//          Copyright(C) 1999-2012, O.Mizno
//
//      Sample of Select plug-in for erasing vertex colors of selected
//     vertices.
//      Created DLL must be installed in "Plugins\Select" directory.
//
//    　選択頂点の頂点カラーを消去する選択プラグインのサンプル。
//    　作成したDLLは"Plugins\Select"フォルダに入れる必要がある。
//
//
//---------------------------------------------------------------------------

#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "MQPlugin.h"

BOOL EraseColor(MQDocument doc);


//---------------------------------------------------------------------------
//  DllMain
//---------------------------------------------------------------------------
BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					 )
{
	//プラグインとしては特に必要な処理はないので、何もせずにTRUEを返す
    return TRUE;
}

//---------------------------------------------------------------------------
//  MQGetPlugInID
//    プラグインIDを返す。
//    この関数は起動時に呼び出される。
//---------------------------------------------------------------------------
MQPLUGIN_EXPORT void MQGetPlugInID(DWORD *Product, DWORD *ID)
{
	// プロダクト名(制作者名)とIDを、全部で64bitの値として返す
	// 値は他と重複しないようなランダムなもので良い
	*Product = 0x56A31D20;
	*ID      = 0xF0567AC5;
}

//---------------------------------------------------------------------------
//  MQGetPlugInName
//    プラグイン名を返す。
//    この関数は[プラグインについて]表示時に呼び出される。
//---------------------------------------------------------------------------
MQPLUGIN_EXPORT const char *MQGetPlugInName(void)
{
	// プラグイン名
	return "Erase vertex color           Copyright(C) 1999-2012, O.Mizno";
}

//---------------------------------------------------------------------------
//  MQGetPlugInType
//    プラグインのタイプを返す。
//    この関数は起動時に呼び出される。
//---------------------------------------------------------------------------
MQPLUGIN_EXPORT int MQGetPlugInType(void)
{
	// 選択部変形用プラグインである
	return MQPLUGIN_TYPE_SELECT;
}

//---------------------------------------------------------------------------
//  MQEnumString
//    ポップアップメニューに表示される文字列を返す。
//    この関数は起動時に呼び出される。
//---------------------------------------------------------------------------
MQPLUGIN_EXPORT const char *MQEnumString(int index)
{
	switch(index){
	case 0: return "Erase vertex color";
	}
	return NULL;
}

//---------------------------------------------------------------------------
//  MQModifySelect
//    メニューから選択されたときに呼び出される。
//---------------------------------------------------------------------------
MQPLUGIN_EXPORT BOOL MQModifySelect(int index, MQDocument doc)
{
	switch(index){
	case 0: return EraseColor(doc);
	}
	return FALSE;
}

//頂点カラーの消去
BOOL EraseColor(MQDocument doc)
{
	BOOL result = FALSE;

	int obj_count = doc->GetObjectCount();

	// オブジェクトの数だけ繰り返し
	for(int obj_idx=0; obj_idx<obj_count; obj_idx++)
	{
		MQObject obj = doc->GetObject(obj_idx);
		if(obj == NULL)
			continue;

		// オブジェクト中の面数(正確には配列の大きさ)を得る
		int face_count = obj->GetFaceCount();

		for(int face_idx=0; face_idx<face_count; face_idx++)
		{
			// 無効な面(頂点数が0)ならパス
			int vert_count = obj->GetFacePointCount(face_idx);
			if(vert_count == 0)
				continue;

			// 面内の頂点ごとに処理
			int vertex[4];
			obj->GetFacePointArray(face_idx, vertex);
			for (int vert_idx=0; vert_idx<4; vert_idx++){
				// 頂点が選択されていれば、頂点カラーを白色不透明に
				if(doc->IsSelectVertex(MQSelectVertex(obj_idx,vertex[vert_idx])))
				{
					obj->SetFaceVertexColor(face_idx, vert_idx, 0xFFFFFFFF);
					result = TRUE;
				}
			}
		}
	}
	
	return result;
}

