﻿//---------------------------------------------------------------------------
//
//   Wired
//
//          Copyright(C) 1999-2012, O.Mizno
//
//     Sample for Object plug-in for deleting faces with reserving lines
//    in the faces.
//     Created DLL must be installed in "Plugins\Object" directory.
//
//    　オブジェクト中の辺のみを残して面を削除するオブジェクトプラグイン
//    のサンプル。
//    　作成したDLLは"Plugins\Object"フォルダに入れる必要がある。
//
//
//---------------------------------------------------------------------------

#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "MQPlugin.h"

BOOL Wired(MQDocument doc);
BOOL DeleteLines(MQDocument doc);


//---------------------------------------------------------------------------
//  DllMain
//---------------------------------------------------------------------------
BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					 )
{
	//プラグインとしては特に必要な処理はないので、何もせずにTRUEを返す
    return TRUE;
}

//---------------------------------------------------------------------------
//  MQGetPlugInID
//    プラグインIDを返す。
//    この関数は起動時に呼び出される。
//---------------------------------------------------------------------------
MQPLUGIN_EXPORT void MQGetPlugInID(DWORD *Product, DWORD *ID)
{
	// プロダクト名(制作者名)とIDを、全部で64bitの値として返す
	// 値は他と重複しないようなランダムなもので良い
	*Product = 0x56A31D20;
	*ID      = 0x157E8FB0;
}

//---------------------------------------------------------------------------
//  MQGetPlugInName
//    プラグイン名を返す。
//    この関数は[プラグインについて]表示時に呼び出される。
//---------------------------------------------------------------------------
MQPLUGIN_EXPORT const char *MQGetPlugInName(void)
{
	// プラグイン名
	return "Wired           Copyright(C) 1999-2012, O.Mizno";
}

//---------------------------------------------------------------------------
//  MQGetPlugInType
//    プラグインのタイプを返す。
//    この関数は起動時に呼び出される。
//---------------------------------------------------------------------------
MQPLUGIN_EXPORT int MQGetPlugInType(void)
{
	// オブジェクト変形用プラグインである
	return MQPLUGIN_TYPE_OBJECT;
}

//---------------------------------------------------------------------------
//  MQEnumString
//    ポップアップメニューに表示される文字列を返す。
//    この関数は起動時に呼び出される。
//---------------------------------------------------------------------------
MQPLUGIN_EXPORT const char *MQEnumString(int index)
{
	switch(index){
	case 0: return "Wired";
	case 1: return "Delete lines";
	}
	return NULL;
}

//---------------------------------------------------------------------------
//  MQModifyObject
//    メニューから選択されたときに呼び出される。
//---------------------------------------------------------------------------
MQPLUGIN_EXPORT BOOL MQModifyObject(int index, MQDocument doc)
{
	switch(index){
	case 0: return Wired(doc);
	case 1: return DeleteLines(doc);
	}
	return FALSE;
}

class EdgeHash
{
public:
	int vertex;
	EdgeHash *next;

	EdgeHash(int v) {
		vertex = v;
		next = NULL;
	}
	virtual ~EdgeHash() {
		if(next != NULL)
			delete next;
	}
	void Add(EdgeHash *hash) {
		if(next != NULL)
			next->Add(hash);
		else
			next = hash;
	}
	EdgeHash *Search(int v)
	{
		if(v == vertex)
			return this;
		if(next != NULL)
			return next->Search(v);
		return NULL;
	}
};

//ワイヤー化
BOOL Wired(MQDocument doc)
{
	int obj_index, face_count, vert_count;
	int face, vert, vert_index[4];
	EdgeHash **edge;

	// カレントオブジェクトの取得
	obj_index = doc->GetCurrentObjectIndex();
	if(obj_index < 0)
		return FALSE;
	MQObject obj = doc->GetObject(obj_index);
	if(obj == NULL)
		return FALSE;

	face_count = obj->GetFaceCount();
	vert_count = obj->GetVertexCount();

	// エッジ用のハッシュを初期化
	edge = (EdgeHash **)malloc(sizeof(EdgeHash *) * vert_count);
	for(vert=0; vert<vert_count; vert++)
		edge[vert] = NULL;

	// 辺のデータを保存しておく
	for(face=0; face<face_count; face++)
	{
		int i,v1,v2;
		int count = obj->GetFacePointCount(face);
		if(count == 0) continue;

		obj->GetFacePointArray(face, vert_index);

		switch(count){
		case 2:
			v1 = min(vert_index[0], vert_index[1]);
			v2 = max(vert_index[0], vert_index[1]);
			if(edge[v1] != NULL) {
				if(edge[v1]->Search(v2) == NULL)
					edge[v1]->Add(new EdgeHash(v2));
			} else
				edge[v1] = new EdgeHash(v2);
			break;
		case 3:
		case 4:
			for(i=0; i<count; i++)
			{
				v1 = min(vert_index[i], vert_index[(i+1)%count]);
				v2 = max(vert_index[i], vert_index[(i+1)%count]);
				if(edge[v1] != NULL) {
					if(edge[v1]->Search(v2) == NULL)
						edge[v1]->Add(new EdgeHash(v2));
				} else
					edge[v1] = new EdgeHash(v2);
			}
			break;
		}
	}

	// ラインを登録
	for(vert=0; vert<vert_count; vert++)
	{
		vert_index[0] = vert;
		for(EdgeHash *ce=edge[vert]; ce!=NULL; ce=ce->next)
		{
			vert_index[1] = ce->vertex;
			obj->AddFace(2, vert_index);
		}
	}

	// オリジナルの面を削除
	for(face=0; face<face_count; face++)
		obj->DeleteFace(face);

	// オブジェクトをコンパクト化
	obj->Compact();

	// ハッシュを削除
	for(vert=0; vert<vert_count; vert++) {
		if(edge[vert] != NULL)
			delete edge[vert];
	}
	free(edge);
	
	return TRUE;
}

//ラインの消去
BOOL DeleteLines(MQDocument doc)
{
	int obj_index, face_count, face;
	char buf[64];

	// カレントオブジェクトの取得
	obj_index = doc->GetCurrentObjectIndex();
	if(obj_index < 0)
		return FALSE;
	MQObject obj = doc->GetObject(obj_index);
	if(obj == NULL)
		return FALSE;

	// オブジェクト中の面数を取得
	face_count = obj->GetFaceCount();

	int count = 0;
	for(face=0; face<face_count; face++)
	{
		// 指定した面は2つの頂点によって構成される、つまりラインであれば
		// 消去する
		if(obj->GetFacePointCount(face) == 2){
			obj->DeleteFace(face);
			count++;
		}
	}

	if(count == 0)
	{
		MessageBox(MQ_GetWindowHandle(), "No lines in current object.",
			"Delete lines", MB_OK|MB_ICONINFORMATION);
		return FALSE;
	}

	obj->Compact();

	sprintf_s(buf, sizeof(buf), "%d lines deleted.", count);
	MessageBox(MQ_GetWindowHandle(), buf, "Delete lines", MB_OK|MB_ICONINFORMATION);
	
	return TRUE;
}

